/*
  Test message concept

  Each message will be of the following format
  
  MsgId  byte    Just something made up, maybe lower nibble is length
  Len    byte    Number of bytes remaining in message after this byte including chksum byte
  Data0  byte    Data bytes what they mean is message dependent
   .
   .
  Datan  byte
  Checksum       Checksum byte, the 2's complement of the sum of all bytes except MsgId
  

  Two simple messages for this test

  A single data byte to simulate switch closure or a key press
  MsgId = 0xA1
  Len   = 2
  Databyte
  CheckSum

  Send six bytes to be processed 
  MsgId = 0xA6
  Len = 7
  Six Data bytes
  CheckSum
  
*/

#include "simpletools.h"                      // Include simple tools
#include "fdserial.h"

#define MAXCHARS 78

/* Parsing the incoming messages requires a state machine driven by
   each incoming byte from the serial port
   With the following states
*/ 

#define WAIT4MSGID 1      //Get bytes until we see one we know
#define GETLENBYTE 2      //Get the number of bytes remaining in message
#define GETDATABYTES 3    //Get the data bytes. Len byte - 1 bytes
#define GETCHECKSUM 4     //Get checksum byte

#define MAXMSGSIZE 16     //Small messages for now
 
   
                         
// Connections for Activity Bot Board
int RX = 10, TX = 9;       // XBEEwifi connected pins

fdserial *xbwifi;        // Serial device to talk to XBEEwifi

// An array to hold an incoming message of max length
uint8_t MsgBuffer[MAXMSGSIZE];

// Message parsing variables
uint8_t rxchar;
uint8_t MyChkSum;
uint8_t BValue;
uint8_t *pRxMsgPtr;
uint8_t *pMsgPtr;
uint8_t bytecount;
uint8_t bytesneeded;
int     RxMsgState;

//Create an array to store 6 byte message values for testing
uint8_t Sixbytes[6];


/* For testing of the receive message function. Just define it here
   for later version launch it into it's own cog to watch for messages
   and handshake with main line code when message is available
*/
void RxMessage() {
uint8_t chktest;
uint8_t idx;      //Stand alone version only
int ch;

// HERE IN STAND ALONE TEST VERSION
int msgflag;      // Set to 1 to build a new message
                  // State Machine sets it to 0 indicating have a message
                  // When in seperate cog handshake flag will be used

  //Get a char from serial port wait for it to arrive
  //In this test version it will hang the system
  RxMsgState = WAIT4MSGID;  // HERE IN STAND ALONE TEST VERSION
  msgflag = 1;              // HERE IN STAND ALONE TEST VERSION
  idx = 0;                  // HERE IN STAND ALONE TEST VERSION
 
 print("RxMessage Looking\n");
 while(msgflag == 1) {

  rxchar = fdserial_rxChar(xbwifi);
  // Execute state machine on rxchar
  switch (RxMsgState) {
   case WAIT4MSGID:
        if ( (rxchar == 0xA1) || (rxchar == 0xA6)) {
          print("\nGot Message %x\n",rxchar);
          MsgBuffer[idx++] = rxchar;
          MyChkSum = 0;
          RxMsgState = GETLENBYTE;
        }
        break;
   case GETLENBYTE:
        // Already pointing at len byte
        MsgBuffer[idx++] = rxchar;
        bytesneeded = (uint8_t)rxchar -1; // Do not include checksum byte
        bytecount = 0;
        MyChkSum = (uint8_t)rxchar;       // First byte of sum
        RxMsgState = GETDATABYTES;
        break;
   case GETDATABYTES:
        //Already pointing at next byte
        MsgBuffer[idx++] = rxchar;
        MyChkSum += (uint8_t)rxchar;
        bytecount++;
        if(bytecount == bytesneeded) {
         RxMsgState = GETCHECKSUM;
        }        
        break;
   case GETCHECKSUM:
        //Now pointing at checksum byte
        MsgBuffer[idx] = rxchar;  // No need to inc to next byte this time
        /*Adding the received checksum byte to MyChkSum should be
          zero if we have a valid checksum because the checksum in message
          is the negitive of the sum
        */
        chktest = (uint8_t)rxchar + MyChkSum;
        chktest = chktest & 0xFF;  //Only 8 bits wanted
        if(chktest == 0) {
         msgflag = 0;  // Were done have valid message
         RxMsgState = WAIT4MSGID; // Redundent in stand alone version 
        } 
        fdserial_rxFlush(xbwifi);               // Clear the serial buffer
        break;
  }

 }// End while msgflag

}

int main()   // Main function
{
uint8_t ix;  //Index into MsgBuffer array
int ch;

  // Add startup code here.
  xbwifi = fdserial_open(RX,TX,0,9600);   // Full Duplex serial to XBEE
  fdserial_rxFlush(xbwifi);               // Clear the serial buffer
  pause(100);

  RxMsgState = WAIT4MSGID;


 
  while(1)
  {
    // Add main loop code here.


    //Get a valid message, waits forever for one
    RxMessage();

    //Process the valid message
    //Have plenty of time not watching for new message 
    //until done.

    // Start at beginning of message
    ix = 0;
    
    //Use a case statement to process each type of message
    switch (MsgBuffer[ix++]) {
     case 0xA1:
       print("Message A1\n");
       ix++;       // Bump index past len byte of message
       BValue = MsgBuffer[ix];
       //Use the byte as if it was a keypress command
       switch (BValue) {
        case 1:
         high(26); // Turn on P26 Led
         print("P26 ON\n");
         break;
        case 2:
         low(26);  // Turn off P26 Led
         print("P26 OFF\n");
         break;
        case 4:
         high(27); // Turn on P27 Led
         print("P27 ON\n");
         break;
        case 8:
         low(27); // Turn off P27 Led
         print("P27 OFF\n");
         break;
       }
       break;
     case 0xA6:
       print("Message A6\n");
       ix++;       // Bump index past len byte of message
       for(int b=0;b<6;b++) {
        Sixbytes[b] = MsgBuffer[ix++];
       }
       for(int br=0;br<6;br++){
         print("$%02x ",Sixbytes[br]);
       }
       print("\n");     
       break;
   }// end switch message
    
  }  
}
